##
# get_exten
#
# get the location of an extension
# based on the internal or extenral.conf
# Extracted from old getexten.agi
#
# Felix Nilam - 08/02/2009
##
sub get_exten
{
	# grab the extension
	my $exten = shift;        
	if ($exten !~ /^\d+$/)
	{
		return 0;
	}
	my $server = shift;
	my $prefix = "";
	if ( $server )
	{
		$prefix = "/data/$server";
	}

	#  The extension info we need may live in either the internal or external context
	my $return;
	$return = check_conf("$prefix/etc/asterisk/fonality/external.conf", $exten, 0);
	if(defined($return))
	{
		return $return;
	}

	$return = check_conf("$prefix/etc/asterisk/fonality/internal.conf", $exten, 0);
	if(defined($return))
	{
		return $return;
	}

	# Felix Nilam - 08/24/2009
	# look into link-internal, link-external and link
	$return = check_conf("$prefix/etc/asterisk/fonality/link-internal.conf", $exten, 1);
	if(defined($return))
	{
		return $return;
	}

	$return = check_conf("$prefix/etc/asterisk/fonality/link-external.conf", $exten, 1);
	if(defined($return))
	{
		return $return;
	}

	$return = check_conf("$prefix/etc/asterisk/fonality/link.conf", $exten, 1);
	if(defined($return))
	{
		return $return;
	}

	# if it reaches this point, extension is not found
	return 0;
}

# Felix Nilam - 08/25/2009
# check_conf
# check the conf file and return the location and linked server
#
# Arguments: conf_file,
#			 extension,
#            link   0 or 1 flag if it is link, link-external or link-internal
# Returns: hasref(location, linked_server) or undef if not found
sub check_conf
{
	my $conf_file = shift;
	my $exten     = shift;
	my $link      = shift;

	open(CONF, "<$conf_file") || die "Can't read $conf_file: $!";
	
	# split the extension into individual digits
	my @digits = split(//, $exten);
	# generate the pattern matching
	my $pattern = '';
	foreach my $dig (@digits)
	{
		$pattern .= '['.$dig.'X]';
	}
	
	while (<CONF>)
	{
		if($link == 1)
		{
			if (/^exten => _$pattern,1,(Macro\([^,]+,[^,]+,[^,]+,[^),]+\))/)
			{
				close CONF;

				# split the macro
				my $macr = $1;
				$macr =~ s/^Macro\(//g;
				$macr =~ s/\)$//g;
				my @macro = split(/,/, $macr);
				
				return $macro[2];
			}
		}
		else
		{
			if (/^exten => $exten,1,Macro\([^,]+,[^,]+,[^,]+,([^),]+)/) 
			{
				close CONF;
				return $1;
			}
		}
	}
	close CONF;
	return undef;
}

1;
