use XML::Simple;

my $name_map = {};

my $license_perm_map = {};

sub get_permissions
{
	my $server_id = shift;
	my $prefix;
	if ( $server_id )
	{
		$prefix = "/data/$server_id";
	}
	# take in permissions xml
	my $xml = new XML::Simple;
	my $data = $xml->XMLin("$prefix/etc/asterisk/hud3.0/p3.xml");
	# this is required for uniformity between single and mutli-linked servers - Andre
	if(ref($data->{permissions}) eq "HASH") {
		$data->{permissions} = [$data->{permissions}];
	}

	# Felix Nilam - 11/16/2009
	# get the user license
	my $user_data = $xml->XMLin("$prefix/etc/asterisk/hud3.0/u3.xml");
	if(ref($user_data->{server}) eq "HASH") {
		$data->{users} = [$user_data->{server}];
	}

	# get the feature to permissions mapping
	my $feature_data = $xml->XMLin("$prefix/etc/asterisk/hud3.0/f3.xml");
	if(ref($feature_data->{feature}) eq "HASH") {
		my $license_to_perm;
		my $feature_components = $feature_data->{feature};
		foreach my $fc (keys %{$feature_components})
		{
			my $name = $fc;
			my $perm = $feature_components->{$fc}->{permission};
			if($perm && ref($perm) ne "HASH"){
				$license_to_perm->{$name} = $perm;
			}
		}

		foreach my $fc (keys %{$license_to_perm})
		{
			my @p = split(",", $license_to_perm->{$fc});
			foreach my $pe (@p)
			{
				if($license_perm_map->{$pe})
				{
					$license_perm_map->{$pe} .= ",".$fc;
				}
				else
				{
					$license_perm_map->{$pe} = $fc;
				}
			}
		}
	}

	return $data;
}

sub get_ast_hdrs
{
	my %info;

	# get asterisk headers
	while (<STDIN>)
	{
		$info{$1} = $2 if /^(.*): (.*)$/;
		dbg("$1 = $2");

		last if /^\r?\n/;
	}
	$info{agi_channel} =~ s/-.*$//;

	return %info;
}


sub ssend
{
	my $data = shift;
	print STDOUT "$data\n";
	my $line = <STDIN>;

	return $line;
}

# debug command
sub dbg
{
	my $log = shift;
	chomp($log);
	open F, ">>/tmp/agi.log";
	print F localtime() . ": $log\n";
	close(F);
}

# returns the CallerID (extension number) of the person paging
sub get_cid
{
	my $info = shift;

	my $cid = $info->{agi_callerid};
	$cid =~ s/^.*<(\d+)>$/$1/;

	if (!$cid)
	{
		dbg("no valid callerid");
	}

	return $cid;
}

# verify user has proper rights to perform function
sub check_rights
{
	my ($ref, $ext, $exts, $right_name, $must_be_group, $must_be_individual) = @_;

	set_name_map($ref);
	
	# find out who's allowed
	my %allowed;
	# we need to loop through multiple servers, possibly - Andre
	foreach my $srv (@{$ref->{permissions}}) {
		%allowed = (%allowed, map { $_ => 1 } get_exts_from_group($srv, $ext, $right_name));
	}
	
	# Felix Nilam - 11/16/2009
	# check user license
	my %allowed2;
	if($license_perm_map->{$right_name})
	{
		foreach my $srv (@{$ref->{users}})
		{
			$allowed2{$_} = 1 for map { get_exts_from_user($_, $license_perm_map->{$right_name}) } @{$ref->{users}};
		}
	}

	# make sure we have permission
	foreach (keys %{$exts})
	{
		
		unless (
			($must_be_group      && $exts->{$_} == 1 && $allowed{$_} && $allowed2{$_}) ||
			($must_be_individual && $exts->{$_} == 0 && $allowed{$_} && $allowed2{$_}) ||
			(!$must_be_group && !$must_be_individual && $allowed{$_} && $allowed2{$_})
		)
		{
			# not allowed
			return 0;
		}
	}

	return 1;
}

sub set_name_map
{
	my $ref = shift;
	
	# map group ids to names, unfortunately there is ID uncomformity in the p3.xml
	# groups are labeled by their names, by a groups' group is labeled by its id - Andre
	foreach my $r (@{$ref->{permissions}}) {
		my $pos = $r->{group};
		my $one = exists $pos->{name};	
		foreach my $key (keys %{$pos}) {
			if($one) {
				$name_map{$pos->{id}} = $pos->{name};
				last;
			} else {
				$name_map{$pos->{$key}->{id}} = $key;
			}
		}
	}
}

# get extensions within allowed groups
sub get_exts_from_group
{
	my ($ref, $ext, $right_name) = @_;

	my $ext_in_group = 0;
	# which groups have this right?
	my (@groups, $loc, $type, $one, $members);
	my $pos = $ref->{group};
	
	$one = exists $pos->{name};
	foreach my $key (keys %{$pos}) {
		# if there is a key 'name', it implies only 1 group`
		# this will break if someone has only 1 group called 'name'. Likelyhood is low,
		# since a user cannot delete group 'Basic Human Rights' - Andre
		if($one) {
			$loc = $pos->{permissions};
			$members = $pos->{members}->{ext};
		} else {
			$loc = $pos->{$key}->{permissions};
			$members = $pos->{$key}->{members}->{ext};
		}
		
		# is the ext that is committing this act of pemission exist in this group?
		foreach my $mem (split(",",$members)) {
			if($mem == $ext) {
				$ext_in_group = 1;
				last;
			}
		}
		
		next unless($ext_in_group);
		my @permissions = ref($loc) eq "ARRAY" ? @$loc : ($loc);
		# search through each group's groups
		foreach (@permissions) {
			# search through the permissions associated
			foreach my $perm (split(",",$_->{ids})) {
				# is it there? # add the name of the group
				if($perm eq $right_name) {
					push(@groups, $name_map{$_->{group}});
					last;
				}
			}
		}
		
		$ext_in_group = 0;
		last if $one;
	}	
	
	# get the extensions for each group
	return map { get_ref($ref, "group" . (!$one ? "->$_" : "") . "->members->ext") } @groups;
}

# get extensions from users with allowed feature components
sub get_exts_from_user
{
	my ($ref, $feature_name) = @_;
	my @ret;
	foreach my $u (keys %{$ref->{user}})
	{
		if(exists $ref->{user}->{$u}->{license})
		{
			my @fc = split(",",$ref->{user}->{$u}->{license});
			foreach my $feature (@fc)
			{
				if(grep { $_ eq $feature } split(",",$feature_name))
				{
					push(@ret, $ref->{user}->{$u}->{extension}->{number});
					last;
				}
			}
		}
	}
	return map { $_ => 1} @ret;
}

# sweet function to return a list of shiz from refs
sub get_ref
{
	my ($ref, @list) = @_;

	@list = map { split("->") } @list;
	
	my $var = $ref;
	foreach (@list)
	{
		if (ref($var) eq "HASH")
		{
			$var = $var->{$_};
		}
		else
		{
			$var = undef;
			last;
		}
	}

	my @list = ref($var) eq "ARRAY" ? @{$var} : ($var);
	foreach (@list)
	{
		s/R\((\d+),(\d+)\)/join(",", $1 .. $2)/eg;
	}
	return map { split(",") } @list;
}

# produce a list of extensions from extensions/extension groups
# returns a hash of extensions => 1 if in a group, => 0 if not, group extensions are removed
sub get_exts
{
	my ($ref, @exts) = @_;

	my %out;
	foreach my $srv (@{$ref->{permissions}}) {
		%out = (%out, %{_get_exts($srv, @exts)});
		# _get_exts will remove a group extension from the structure it returns, but
		# that same extension is never removed from @exts, so it will possibly appear
		# as $out{group extension} = 0, if iterating over mutliple servers, which is bad.
		# group extensions need to be removed.
		# If the ext does not exist in %out, remove it from @exts - Andre
		for(my $i = 0; $i < @exts; $i++) {
			unless(exists $out{$exts[$i]}) {
				splice(@exts, $i, 1);
				$i--;
			}
		}
	}

	return \%out;
}

sub _get_exts
{
	my ($ref, @exts) = @_;
	$ref ||= {};
	
	my $pos = $ref->{group} || {};
	my (%struct, $loc);
	
	# assume none of the exts are group extensions
	foreach my $ext (@exts) {
		$struct{$ext} = 0;
	}
	
	my $one = exists $pos->{name};	
	foreach my $key (keys %{$pos}) {
		# if there is a key 'name', it implies only 1 group`
		# this will break if someone has only 1 group called 'name'. Likelyhood is low,
		# since a user cannot delete group 'Basic Human Rights' - Andre
		my $inner_pos;
		if($one) {
			# is this a requested group extension?
			next unless(exists $struct{$pos->{ext}});
			delete $struct{$pos->{ext}};
			$inner_pos = $pos->{members};
		} else {
			# is this a requested group extension?
			next unless(exists $struct{$pos->{$key}->{ext}});
			delete $struct{$pos->{$key}->{ext}};
			$inner_pos = $pos->{$key}->{members};
		}
		
		if(ref($inner_pos) eq "ARRAY") {
			foreach my $set (@{$inner_pos}) {
				foreach (get_ref($set, "ext")) {
					$struct{$_} = 1;
				}
			}
		} else {
			foreach (get_ref($inner_pos, "ext")) {
				# add all extensions assoicated with this group
				$struct{$_} = 1;
			}
		}

		last if $one;
	}

	return \%struct;
}

1;
